use platform::EventInterface;

pub mod geom;
pub mod layout;
pub mod platform;
pub mod component;
pub mod widget;
pub mod input;

pub mod prelude {

}

pub struct UI<PT: platform::PlatformTuple> {
    layout_cache: std::rc::Rc<layout::LayoutCache>,
    platform: PT,
}

impl<PT: platform::PlatformTuple> UI<PT> {
    pub fn new(pt: PT) -> Self {
        Self {
            layout_cache: layout::LayoutCache::new(),
            platform: pt,
        }
    }

    pub fn update(&mut self) {
        while let Some(evt) = self.platform.event_mut().poll_event() {
            match evt {
                platform::event::WindowEvent::CloseRequest => {
                    std::process::exit(0);
                }
                _ => {},
            }
        }
    }

    pub fn sink_event(&mut self, evt: platform::event::UIEvent) {
        
    }

    pub fn run(mut self) {
        loop {
            self.update();

            self.platform.event_mut().wait_event();
        }
    }
}
