pub mod render;

pub trait RenderInterface {}

pub mod event;

pub trait EventInterface {
    fn poll_event(&mut self) -> Option<event::WindowEvent>;
    fn wait_event(&mut self);
}

pub trait TextInterface {}

pub trait PlatformTuple {
    type Render: RenderInterface;
    type Event: EventInterface;
    type Text: TextInterface;

    fn render(&self) -> &Self::Render;
    fn render_mut(&mut self) -> &mut Self::Render;

    fn event(&self) -> &Self::Event;
    fn event_mut(&mut self) -> &mut Self::Event;

    fn text(&self) -> &Self::Text;
    fn text_mut(&mut self) -> &mut Self::Text;
}

impl<RI: RenderInterface, EI: EventInterface, TI: TextInterface> PlatformTuple for (RI, EI, TI) {
    type Render = RI;
    type Event = EI;
    type Text = TI;

    fn render(&self) -> &Self::Render {
        &self.0
    }
    fn render_mut(&mut self) -> &mut Self::Render {
        &mut self.0
    }

    fn event(&self) -> &Self::Event {
        &self.1
    }
    fn event_mut(&mut self) -> &mut Self::Event {
        &mut self.1
    }

    fn text(&self) -> &Self::Text {
        &self.2
    }
    fn text_mut(&mut self) -> &mut Self::Text {
        &mut self.2
    }
}
