use crate::geom::IPoint;

#[derive(Clone, Copy, Debug)]
pub struct ButtonSet(u8);

pub struct MouseState {
    pub pos: IPoint,
    pub buttons: ButtonSet,
    pub last_buttons: ButtonSet,
}

impl MouseState {
    pub fn pressed(&self) -> ButtonSet {
        ButtonSet(self.buttons.0 & !self.last_buttons.0)
    }

    pub fn released(&self) -> ButtonSet {
        ButtonSet(self.last_buttons.0 & !self.buttons.0)
    }
}

#[derive(Clone, Debug)]
pub enum UIEvent {
    MouseUpdate,
    KeyDown,
    KeyUp,
    TextInput,
}

#[derive(Clone, Debug)]
pub enum WindowEvent {
    RepaintRequest,
    Resize,
    CloseRequest,
}
