#[derive(Clone, Copy, Debug)]
pub enum SurfaceFormat {
    RGBA,
    ARGB,
    ABGR,
    RGB,
    GBR,
}

impl SurfaceFormat {
    pub fn stride(&self) -> usize {
        match self {
            Self::RGBA | Self::ARGB | Self::ABGR => 4,
            Self::RGB | Self::GBR => 3,
        }
    }

    /// Red colour value bitmask, expressed as (bit-index, len)
    pub const fn red_mask(&self) -> (u8, u8) {
        match self {
            Self::RGBA => (0, 8),
            Self::RGB => (0, 8),
            _ => todo!(),
        }
    }

    /// Green colour value bitmask, expressed as (bit-index, len)
    pub const fn green_mask(&self) -> (u8, u8) {
        match self {
            Self::RGBA => (8, 8),
            Self::RGB => (8, 8),
            _ => todo!(),
        }
    }

    /// Blue colour value bitmask, expressed as (bit-index, len)
    pub const fn blue_mask(&self) -> (u8, u8) {
        match self {
            Self::RGBA => (16, 8),
            Self::RGB => (16, 8),
            _ => todo!(),
        }
    }

    pub const fn alpha_mask(&self) -> (u8, u8) {
        match self {
            Self::RGBA => (24, 8),
            Self::RGB => (0, 0),
            _ => todo!(),
        }
    }
}

#[derive(Clone, Copy, Debug)]
pub enum AccessOrder {
    RowMajor,
    ColumnMajor,
}

pub trait Surface {
    fn width(&self) -> usize;
    fn height(&self) -> usize;

    fn access(&self) -> AccessOrder;
    fn format(&self) -> SurfaceFormat;
}
