use std::rc::{Rc, Weak};

use crate::{component::Component, input::InputState, layout::LayoutNodeAccess};

#[derive(Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash, Debug)]
pub struct WidgetID(usize);
static NEXT_WIDGET_ID: std::sync::atomic::AtomicUsize = std::sync::atomic::AtomicUsize::new(1);
impl WidgetID {
    pub fn new() -> Self {
        Self(
            NEXT_WIDGET_ID.fetch_add(1, std::sync::atomic::Ordering::SeqCst)
        )
    }
}

pub trait Widget<C: Component>: 'static {
    fn poll(&mut self, input_state: &InputState) -> impl Iterator<Item = C::Msg>;

    fn layout_node(&self) -> &dyn LayoutNodeAccess;
    fn render(&self, surface: ());
}
